/*
 *	wnv.h
 *
 *	Header file for the Winnov Videum drivers.
 *
 *	This program is copyright 1998, 1999
 *	by Bill Dirks
 *	Some code copyright by Winnov, L.P., used by permission.
 *
 *	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version
 *	2 of the License, or (at your option) any later version.
 */

/*	The maximum number of boards per system. You can increase this
 *	to support more cards per system
 */
#define WNV_NBOARDS		4

/*
 *	These are all the Winnov WAVI chip registers, register bits,
 *	register fields, etc.
 */

#define WAVI_BOARD0CONFIG	0x5A04
#define WAVI_BOARD1CONFIG	0x5A05

#define WAVI_REMOTEBOARD0CONFIG    0x7A04
#define WAVI_REMOTEBOARD1CONFIG    0x7A05
#define WAVI_PCICONFIG0      0x00D8
#define WAVI_PCICONFIG1      0x00D9

#define WAVI_PORTDESELECT	0x0000
#define WAVI_PORT1SELECT	0x0001
#define WAVI_PORT2SELECT	0x0002
#define WAVI_PORT3SELECT	0x0003

#define WAVI_PORT0SELECTMEDIAPLUG        0x16
#define WAVI_PORT1SELECTMEDIAPLUG        0x15
#define WAVI_PORT2SELECTMEDIAPLUG        0x14
#define WAVI_PORT3SELECTMEDIAPLUG        0x12
#define WAVI_PORT4SELECTMEDIAPLUG        0x11
#define WAVI_PORT5SELECTMEDIAPLUG        0x10

#define WAVI_ADLIB		0x0040
#define WAVI_BOARD0PORTDESELECT	0xFFFF
#define WAVI_BOARD0PORT1	0x0160
#define WAVI_BOARD0PORT2	0x0150
#define WAVI_BOARD0PORT3	0x0140
#define WAVI_BOARD1PORT1	0x0120
#define WAVI_BOARD1PORT2	0x0110
#define WAVI_BOARD1PORT3	0x0100

#if 0
#define CPB_HEADER		0
#define CPB_AUDPTR		1
#define CPB_AUDATTN		2
#define CPB_AUDGAIN		3
#define CPB_AUDMIX1		4
#define CPB_AUDMIX2		5
#define CPB_6			6
#define CPB_7			7
#define CPB_VID1PTR		8
#define CPB_VID2PTR		9
#define CPB_RLDPTR1		10
#define CPB_RLDPTR2		11
#define CPB_SPLPTR		12
#define CPB_D			13
#define CPB_E			14
#define CPB_DEFAULT		15
#endif

// Media Plug Local registers
#define WAVI_CMD		0
 #define MP_RESET_HW		0		// remote
 #define MP_END_STREAM		1
 #define MP_WRITE_REG		2
 #define MP_READ_REG		3
 #define MP_WRITE_STREAM	4
 #define MP_READ_STREAM		7
 // read
 #define MP_TIMEOUT		0x4000
 #define MP_DIRTY		0x8000
//void mpCommand(WORD wCommand);

#define WAVI_LCMD		2
 #define MP_WATCHDOG_DISABLE	1
 #define MP_REMOTE_POWER	2
 #define MP_CABLE_CLOCK         4
 #define MP_CHIP_CLOCK		8
 // read
 #define MP_CABLE_DETECT	0x80

#define WAVI_DATA		4

/*
 *	WAVI registers, index numbers passed to the register access functions
 */
#define WAVI_HST1PTR		0
#define WAVI_HST2PTR		1
#define WAVI_SPLPTR		2
#define WAVI_AUDPTR		3
#define WAVI_VID1PTR		4
#define WAVI_MXRPTR		5
#define WAVI_VID2PTR		6
#define WAVI_MXWPTR		7
#define WAVI_HST1DAT		8
#define WAVI_HST2DAT		9
#define WAVI_RLDPTR1		10
#define WAVI_RLDPTR2		11
#define WAVI_MMA		12
#define WAVI_CTL		13
#define WAVI_CTL1		13
#define WAVI_ISR		14
#define WAVI_ITR		15
#define WAVI_ATTN		16
#define WAVI_GAIN		17
#define WAVI_FREQ		18
#define WAVI_4C00		19
#define WAVI_AUDMIX1		20
#define WAVI_AUDMIX2		21
#define WAVI_5800		22
#define WAVI_5C00		23
#define WAVI_VOF		24
#define WAVI_VCT		25
#define WAVI_6800		26
#define WAVI_CTL2		26
#define WAVI_6C00		27
// #define WAVI_CTLL		28
#define WAVI_PTRB		28
#define WAVI_7400		29
#define WAVI_MBNK		29
#define WAVI_HWI2C		30
#define WAVI_TSTMODE		31

/*  Convert register number to offset, whatever	*/
#define ISAOFFSET(reg)		((reg)<<10)
#define PNPOFFSET(reg)		((reg)<<2)
#define MPOFFSET(reg)		((reg)<<3)	// in CMD reg

/*  Reload pointer register IDs	*/
#define WAVI_RP_SPLPTR		2
#define WAVI_RP_VID1PTR		4
#define WAVI_RP_VID2PTR		6

/* MMA Register	*/
#define WAVI_NWDBIT		0x8000  /* NOWS Write Disable */
#define WAVI_NRDBIT		0x4000  /* NOWS Read Disable */
#define WAVI_MMAFIELD		0x3E00  /* Memory Map Address */
#define WAVI_MMEBIT		0x0100  /* Memory Map Enable */
#define WAVI_FMFIELD		0x00C0  /* FM config, Read only */
#define WAVI_DSSFIELD		0x0030  /* Disney Sound Source config, Read Only */
#define WAVI_CVXFIELD		0x000C  /* CoVoX speech thing config, Read Only */
#define WAVI_REVFIELD		0x0003  /* WAVIA ASIC Revision, Read Only */

#define WAVI_AVGCENBIT	    0x8000  /* W97 and later: Average Chroma Enable */
#define WAVI_AUDXSEL_INT97  0x4000  /* W97: Crystal selection is internal (using IMI bit) */
#define WAVI_AUDXSEL	    0x4000  /* WIMP: Crystal selection (replaces IMI bit) */
#define WAVI_CS4218BIT	    0x2000  /* W97 and later: 4218 codec configuration */
#define WAVI_SAA7110	    0x1000  /* W97 and later: select SAA7110 or vpx! */
#define WAVI_BANKCS	    0x0800  /* W97 and later: to enable the memory chip select */
#define WAVI_DHYYY	    0x0400  /* W97 and later: Luma only Delta Huffman */
#define WAVI_DHRES	    0x0200  /* W97 and later: Delta Huffman Residual alogorithm */
#define WAVI_DHRND	    0x0100  /* W97 and later: Delta Huffman Rounding algorithm */
#define WAVI_MEDREN	    0x0080  /* W97 and later: Median Filter Return Effect */
#define WAVI_VIDVCI	    0x0040  /* W97 and later: DVCI video format */
#define WAVI_VERSIONFIELD   0x0030  /* W95 and later: Read Only Version */
#define WAVI_WAVIIDFIELD    0x000F  /* All Versions: WAVI ASIC Revision, Read Only */
#define WAVI_FULLID	    0x003F  /* completely identify the chip version...  W95 and above! */

#define WAVI_WAVI1           0x0002  /* The original Samsung */
#define WAVI_WAVI95          0x0012  /* Hundai part with line avg during Huffman*/
#define WAVI_WAVI97          0x0022
#define WAVI_WIMP            0x0003

/* CTL Register	*/
#define WAVI_VSEBIT	  	0x8000  /* Video Sampler Enable */
#define WAVI_VSTBIT	  	0x4000  /* Video Status */
#define WAVI_VSSBIT	  	0x2000  /* Video Source Select */
#define WAVI_VSCBIT	  	0x1000  /* Video Source Component(1)/Composite(0) */
#define WAVI_RGBBIT	  	0x0800  /* RGB(1)/YUV(0) Video Space */
#define WAVI_VFSFIELD	 	0x0700  /* Video Filter Select */
#define WAVI_VFS_NONE		0x0000  /* Bypass filters */
#define WAVI_VFS_MED3		0x0100  /* 3 tap median filter */
#define WAVI_VFS_FIR2		0x0200  /* 2 pix FIR filter */
#define WAVI_VFS_BOTH2		0x0300  /* Median and 2 pix FIR */
#define WAVI_VFS_FIR4		0x0400  /* 4 pix FIR filter */
#define WAVI_VFS_BOTH4		0x0500  /* Median and 4 pix FIR */
#define WAVI_VFS_FIR8		0x0600  /* 8 pix FIR filter */
#define WAVI_VFS_BOTH8		0x0700  /* Median and 8 pix FIR filter */
#define WAVI_VDFFIELD		0x00E0  /* Video Data Capture Format */
#define WAVI_VDF_DYCQ		0x0000  /* DYCQ */
#define WAVI_VDF_YCQ		0x0020  /* YCQ */
#define WAVI_VDF_YCH		0x0040  /* YCH */
#define WAVI_VDF_RGB		0x0060  /* RGB */
#define WAVI_VDF_DH4		0x0080  /* DH4 */
#define WAVI_VDF_DH5		0x00A0  /* DH5 */
#define WAVI_VDF_DH6		0x00C0  /* DH6 */
#define WAVI_VDF_DH7		0x00E0  /* DH7 */
#define WAVI_CTLBIT4		0x0010  /* not used */
#define WAVI_I2CBIT		0x0008  /* I2C Bus Clock */
#define WAVI_IMIBIT		0x0004  /* IM-Bus Identify */
#define WAVI_IMCBIT		0x0002  /* IM-Bus Clock */
#define WAVI_IMDBIT		0x0001  /* IM-Bus Data */

/* ISR/ITR	*/
#define WAVI_VIEBIT	  	0x8000  /* Video Interrupt Enable */
#define WAVI_VI2BIT	  	0x4000  /* Video Interrupt 2, Read Only */
#define WAVI_VI1BIT	  	0x2000  /* Video Interrupt 1, Read Only */
#define WAVI_AIEBIT	  	0x1000  /* Audio Interrupt Enable */
#define WAVI_AISFIELD	 	0x0C00  /* Audio Interrupt Status, Read Only	*/
#define WAVI_AISFIELD_NOINT	0x0000  /* no audio interrupt */
#define WAVI_AISFIELD_1INT	0x0400  /* one audio interrupt */
#define WAVI_AISFIELD_OVERRUN	0x0800	/* audio overrun */
#define WAVI_AISFIELD_2INT	0x0C00  /* two audio interrupts */
#define WAVI_FIEBIT		0x0200  /* FM Interrupt Enable */
#define WAVI_FMIBIT		0x0100  /* FM Interrupt, Read Only */
#define WAVI_SIEBIT		0x0080  /* SCSI Interrupt Enable */
#define WAVI_SCIBIT		0x0040  /* SCSI Controller Interrupt, Read Only */
#define WAVI_CIEBIT		0x0020  /* Control-L Interrupt Enable */
#define WAVI_CLIBIT		0x0010  /* Control-L Interrupt, Read Only */
#define WAVI_IDTBIT		0x0008  /* Interrupt Detect, Read Only */
#define WAVI_ITMBIT		WAVI_IDTBIT	/* Interrupt test mode, Write Only */
#define WAVI_ISELFIELD		0x0007  /* Interrupt Select */
#define WAVI_ISELFIELD_NONE	0x0000  /* Interrupts disconnected */
#define WAVI_ISELFIELD_IRQ2	0x0001  /* IRQ2 */
#define WAVI_ISELFIELD_IRQ5	0x0002  /* IRQ5 was reversed with 7*/
#define WAVI_ISELFIELD_IRQ7	0x0003  /* IRQ7 was reversed with 5 */
#define WAVI_ISELFIELD_IRQ10	0x0004  /* IRQ10*/
#define WAVI_ISELFIELD_IRQ11	0x0005  /* IRQ11*/
#define WAVI_ISELFIELD_IRQ12	0x0006  /* IRQ12*/
#define WAVI_ISELFIELD_IRQ15	0x0007  /* IRQ15*/

/* Audio ATTN	*/
#define ATTN_MASK_ASCO		0x0FFF  /* mask of useable bits for ASCO */
#define ATTN_MASK_4218		0x3FFF  /* 4218 mask of useable bits */
#define WAVI_ATTLFFIELD		0x0F00  /* Left Output Channel Attenuation, Write Only */
#define WAVI_ATTRTFIELD		0x00F0  /* Right Output Channel Attenuation, Write Only */
#define WAVI_ATTFIELD		(WAVI_ATTLFFIELD|WAVI_ATTRTFIELD)
// 4218 format of the upper 12 bits:
#define WAVI_4218_ATTLFFIELD 	0x3E00  /* 4218: 5 bit Left Output Channel Attenuation */
#define WAVI_4218_ATTRTFIELD	0x01F0  /* 4218: 5 bit Right Output Channel Attenuation */
#define WAVI_4218_ATTFIELD	(WAVI_4218_ATTLFFIELD|WAVI_4218_ATTRTFIELD)

// common between 4216 and 4218
#define WAVI_ASS2FIELD		0x000C  /* Audio Source Select, Write Only */
#define WAVI_ASS2FIELD_VID1	0x0000
#define WAVI_ASS2FIELD_MIC	0x0004
#define WAVI_ASS2FIELD_VID2	0x0008
#define WAVI_ASS2FIELD_LINE	0x000C

#define WAVI_AFSBIT		0x0002  /* Audio freq select (SPAM only) 0=22.5792,1=24.576 */
#define WAVI_AXEBIT		0x0001  /* Audio expansion enable */

/* Audio GAIN	*/
#define WAVI_MUTBIT		0x0400  /* Mute, Write Only */
#define WAVI_ISLBIT		0x0200  /* Input Select Left channel, Write Only */
			    		/* 0=LIN1, 1=IN2 (ASS2) */
#define WAVI_ISRBIT		0x0100  /* Input Select Right channel, Write Only */
			    		/* 0=LIN1, 1=IN2 (ASS2) */
#define WAVI_GAINLFFIELD	0x00F0  /* Left Input Channel Gain, Write Only */
#define WAVI_GAINRTFIELD	0x000F  /* Right Input Channel Gain, Write Only */
#define WAVI_GAINFIELD		0x00FF

/* Audio FREQ	*/
#define WAVI_AOVFIELD		0xFF00  /* Audio Output Volume */
#define WAVI_ACEBIT		0x0080  /* Audio Converter Enable */
#define WAVI_APEBIT		0x0040  /* Audio Phase Error */
#define WAVI_AMSBIT		0x0020  /* Audio Mode Stereo */
#define WAVI_ADDBIT		0x0010  /* Audio Digital Port Data */
#define WAVI_FREQFIELD		0x000F  /* Audio Frequency */
#define WAVI_FREQFIELD_44100	0x0001
#define WAVI_FREQFIELD_29400	0x0002
#define WAVI_FREQFIELD_22050	0x0003
#define WAVI_FREQFIELD_14700	0x0005
#define WAVI_FREQFIELD_11025	0x0007
#define WAVI_FREQFIELD_08820	0x0009
#define WAVI_FREQFIELD_07350	0x000B

/* Audio MIX 1/2  (write-only)	*/
#define WAVI_MIXRFIELD		0xFF00  /* Right channel output volume */
#define WAVI_MIXLFIELD		0x00FF  /* Left channel output volume */

/* Video Offset	*/
#define WAVI_VOFCRFIELD		0xF000  /* Video Offset Chroma Red */
#define WAVI_VOFCBFIELD		0x0F00  /* Video Offset Chroma Blue */
#define WAVI_VOFYFIELD		0x00F0  /* Video Offset Luma */
#define WAVI_GBCBIT		0x0008  /* Video Input Gray to Binary on Chroma */
#define WAVI_GBYBIT		0x0004  /* Video Input Gray to Binary on Luma */
#define WAVI_VSPBIT		0x0002  /* Video Input VSP Enable */
#define WAVI_VSDBIT		0x0001  /* Video Input VSD Detect, Read Only */
#define WAVI_IFMTBITS		(WAVI_GBCBIT|WAVI_GBYBIT|WAVI_VSPBIT|WAVI_VSDBIT)

/* Video Contrast Reg	*/
#define WAVI_VCTCRFIELD		0xF000  /* Cr contrast */
#define WAVI_VCTCBFIELD		0x0F00  /* Cb Contrast */
#define WAVI_VCTYFIELD		0x00F0  /* Y Contrast */
#define WAVI_VHSDFIELD		0x000F  /* Video Horizontal Synch Delay */

/* Control-L Reg	*/
#define WAVI_STRBIT		0x8000  /* Control-L Start bit */
#define WAVI_TRGBIT		0x4000  /* Control-L Trigger Send */
#define WAVI_ARMBIT		0x2000  /* Control-L Armed, Read Only */
#define WAVI_VVSBIT		0x1000  /* Video Vert Synch, Read Only */
#define WAVI_CLDBIT		0x0800  /* Control-L Direct Data */
#define WAVI_E2BIT		0x0400  /* TBD */
#define WAVI_E1BIT		0x0200  /* TBD */
#define WAVI_E0BIT		0x0100  /* TBD */
#define WAVI_CLDATA		0x00FF  /* Control L Data */

#define WAVI_AUDBUFSIZE		0x2000L	/* size of audio buffer in bytes */

/*  CMD0647 private configuration registers	*/
#define CMD_CONFIG_CNTRL	0x51
#define CMD_CONFIG_CMDTIM	0x52
#define CMD_CONFIG_ARTTIM0	0x53
#define CMD_CONFIG_DRWTIM0	0x54
#define CMD_CONFIG_ARTTIM1	0x55
#define CMD_CONFIG_DRWTIM1	0x56
#define CMD_CONFIG_ARTTIM23	0x57
#define CMD_CONFIG_DRWTIM2	0x58
#define CMD_CONFIG_DRWTIM3	0x5B

/*  Video capture microcode instructions  */
#define UI_DECIMATE_LOWPREC(decimation,clocks)  \
        (((decimation-1)<<10)|(((clocks-1)&3)<<8)|(((clocks-1)>>2)&255))
#define UI_SKIP_LOWPREC(clocks)(UI_DECIMATE_LOWPREC(1,clocks))
#define UI_RVP1         0xF0
#define UI_RVP2         0xF1
#define UI_ITR1         0xF2
#define UI_ITR2         0xF3
#define UI_RVS          0xF4
#define UI_SVS          0xF5
#define UI_SIM          0xF6
#define UI_SVM          0xF7
#define UI_NOP          0xF8
#define UI_WTHS         0xF9
#define UI_WTVA         0xFA
#define UI_WTVB         0xFB
#define UI_WTVF1        0xFC
#define UI_WTVF2        0xFD
#define UI_WTX          0xFE
#define UI_HLT          0xFF


/*  ISA Plug-and-Play ioports */

#define READPORT(rp) ((rp << 2) | 0x03)

//defines
#define  ADDRESS_PORT  		0x279
#define  WRITE_DATA_PORT 	0xa79
#define  RD_DATA		0x00
#define  SERIAL_ISOLATION 	0x01
#define  CONFIG_CONTROL 	0x02
#define		RESET_CSN 	0x4
#define     GOTO_WAIT_FOR_KEY 	0x02
#define     RESET_TO_POWER_UP 	0x01
#define  WAKE 			0x03
#define  RESOURCE_DATA 		0x04
#define  STATUS 		0x05
#define  CSN 			0x06
#define  LOGICAL_DEVICE_NUMBER 	0x07
#define  ACTIVATE 		0x30
#define  IO_RANGE_CHECK 	0x31
#define  MEM_BASEH 		0x40
#define  MEM_BASEL 		0x41
#define  MEM_CTL 		0x42
#define  MEM_LENGTHH 		0x43
#define  MEM_LENGTHL 		0x44
#define  MEM_DESC_INCREMENT 	0x8
#define	 MEM32_BASEHH 		0x76
#define  MEM32_BASEHL 		0x77
#define  MEM32_BASELH 		0x78
#define  MEM32_BASELL 		0x79
#define  MEM32_CTL 		0x7A
#define  MEM32_LENGTHHH 	0x7B
#define  MEM32_LENGTHHL 	0x7C
#define  MEM32_LENGTHLH 	0x7D
#define  MEM32_LENGTHLL 	0x7E
#define  MEM32_DESC_INCREMENT 	0x10
#define  IO_BASEH 		0x60
#define  IO_BASEL 		0x61
#define  IO_DESC_INCREMENT 	0x2
#define  INT_LEVEL 		0x70
#define  INT_TYPE 		0x71
#define  INT_DESC_INCREMENT 	0x2
#define  DMA_SELECT_0 		0x74
#define  DMA_SELECT_1 		0x75

#define  MAX_READPORT 0xA0  // higher base address key!
#define  WINNOV_MFG_ID "WNV"    
#define  WINNOV_PNP_VIDEUM1_ID "WNV0001"
#define  WINNOV_PNP_HUMP1_ID "WNV0002"

//PNP board types!
#define PNP_BOARD_NOT_PNP 0                 
#define PNP_BOARD_PCMCIA 1 
#define PNP_BOARD_VIDEUM1 2
#define PNP_BOARD_LEG_MEDIAPLUG 3
#define PNP_BOARD_CIRRUS1 4 
#define PNP_BOARD_VIDEUM_PCI_IDE 5 
#define PNP_BOARD_MEDIAPLUG_PCI_IDE	6


#define  PNP_MAX_BOARDS 10

/* Passed to pnp_assignresources() */
typedef struct 
{
	u16	wBasePort[8];
	u16	wLength[8];
} RES_INFO;

typedef struct 
{
	char ID[10];
	u32  dwHWIndex;                
	u32  dwDevIndex;
	u16  wStartAddress;
	u16  wEndAddress;
	u16  wLength;   
	u16  wIncrement;
	u32  fRangeCheck;
} PNP_INFO;



/*	
 *	Configuration EEPROM definition for Winnov Videum video 
 *	capture cards
 */

struct EEPROM
{
	__u8	bSignature;	//  Keep this at first byte
#define EEPROM_SIG	0xBD
#define EEPROM_SIG_FAKE	0xFC  //present when Eeprom read from INI file...

	__u8	bBoardType;
	__u8	bMinorType;
	__u8	bSpecial;
	__u32	dwNumber;
	__u32	dwKey;
	__u32	dwHwFlags;
	__u32	dwSwFlags;
	__u8	filler[21];	//  Make structure 128 bytes
	char	szBoardRev[16];
	char	szProduct[30];
	char	szOemName[20];
	char	szSerial[20];
	__u8	bChecksum;
};

/* bBoardType: */
#define	VERSION_LITE		0x80	/* BoardType MSB */

#define TYPE_MOVIEMAN		0
#define TYPE_ODIN2		1
#define	TYPE_SPAM2		2
#define TYPE_HOTSHOT		3
#define TYPE_VIDEUM		4
#define TYPE_ALARIS		5
#define TYPE_VIDEUM10		6
#define	TYPE_WIMPCAM1		7
#define	NBOARDTYPES		8
/* bMinorType: */
#define MINORTYPE_WINNOV	0	/* Winnov configuration on 8-pin */
#define MINORTYPE_PICTURETEL	1	/* PictureTel configuration on 8-pin */
#define MINORTYPE_HITACHI	2	/* Hitachi configuration on 8-pin */
#define MINORTYPE_V2WINNOV	3	/* Videum2 standard configuration */
#define MINORTYPE_AV		4	/* Videum10 AV */
#define MINORTYPE_VO		5	/* Videum10 VO */
/* bSpecial: */
#define	DISABLE_HWCOMPRESS	0x01	/* LSB */
#define	SPECIAL_PROMOTIONAL	0x02
/* dwHwFlags */
#define	HWFLAGS_CS4218    	0x00000001/* using CS4218 audio codec */
#define	HWFLAGS_AUDXSEL		0x00000002/* AUDXSEL for audio xtal select */
#define	HWFLAGS_INV_VSYNC	0x00000004/* If set we need to invert Vsync */
#define	HWFLAGS_VCLK_INV	0x00000008/* Set VCLK_INV bit in CTL2 */
#define	HWFLAGS_WIMPDEVICE	0x00000FF0/* For various WIMP-based devices */
#define	 HWFLAGS_PHILIPS	0x00000010/* Philips WIMP camera */
#define	HWFLAGS_XTAL_8K_NOT_PRESENT	0x00001000/* 24.5760 Mhz xtal absent */
#define	HWFLAGS_XTAL_11K_NOT_PRESENT	0x00002000/* 22.5792 Mhz xtal absent */


/*
 *	Global device data structure
 */


struct videum_device;/* forward reference */

struct video_decoder
{
	int	type;
	int	version;
	int	is_initialized;
	int	i2c_addr1;
	int	i2c_addr2;
	int	ntsc_hskip;
	int	ntsc_vskip;
	int	ntsc_width;
	int	ntsc_height;
	int	ntsc_field_order;
	int	pal_hskip;
	int	pal_vskip;
	int	pal_width;
	int	pal_height;
	int	pal_field_order;
	int	preferred_field;
	int	invert_vsync;
	int	hadjust;
	int	vadjust;

	int	num_inputs;
	int	input;
	__u32	standards;
	__u32	standard;
	__u32	frame_period;
	int	decoder_is_stable;
	__u32	decoder_stable_time;

	/*  Changable method functions helps us support multiple	*/
	/*  different types of video decoders easily			*/
	int	(*initialize)(struct videum_device *dev);
	int	(*set_input)(struct videum_device *dev, int x);
	int	(*set_standard)(struct videum_device *dev, int x);
	int	(*set_vcrmode)(struct videum_device *dev, int x);
	int	(*is_stable)(struct videum_device *dev);
};

#define MAXCONTROLS	4

struct video_source
{
	struct v4l2_input	input;
	int			control[MAXCONTROLS];
	struct v4l2_tuner	tuner;
	int			wavi_vsc;
	int			wavi_vss;
	int			wavi_vhsd;
	int			vcrmode;
};
/*  Indices into the array of video_source's */
#define VSOURCE_MXC		0
#define VSOURCE_COMP		1
#define VSOURCE_SVIDEO		2
#define VSOURCE_TUNER		3
#define VSOURCE_COUNT		4

/*  Microcode programs  */
struct ucode_parms
{
	__u8	*buffer;
	int	source_x;
	int	source_y;
	int	source_width;
	int	source_height;
	int	field;
	int	capture_width;
	int	capture_height;
	int	capture_min_size;
	int	capture_max_size;
	int	dma_line;
	int	int1_line;
	int	int2_line;
	int	flags;
	int	filter;
};
/*  Microcode compile flags	*/
#define COMPFLAG_NOVERTFILTER	0x00000001
#define COMPFLAG_FIELDORDER	0x00000002
#define COMPFLAG_VSDELAY	0x00000004
#define COMPFLAG_AVPRO_DMA	0x00000008
#define COMPFLAG_AVPRO_INT	0x00000010
#define COMPFLAG_HUFFMAN	0x00000020
#define COMPFLAG_MARKERCODE	0x00000040
#define COMPFLAG_EOFINT		0x00000080
#define COMPFLAG_VST_WHEN_DONE	0x00000100

#define UC_SIZE			4096
struct microcode
{
	struct ucode_parms	parm;
	__u8			uc[UC_SIZE];
	int			length;
};

/*  Bus-master scatter list  */
struct scatter_node
{
	__u32	addr;
	__u32	len;
};
#define END_OF_SCATTER_LIST	0x80000000

/*  Image translations  */
struct lookup_rgb24
{
	__u32	u_rgb[256];
	__u32	v_rgb[256];
	__u32	y_rgb[256];
	__u8	sat[1024];
};

struct lookup_dehuff
{
	__u8	decode1[512];	/*  Decode one code */
	__u8	length1[512];	/*  Length of one code */
	__u32	decode4[4096];	/*  Decode four codes */
	__u8	length4[4096];	/*  Length of four codes */
	__u8	dcase[4096];	/*  Case  */
};

struct lookup
{
	int	type;
	int	size;
	union
	{
		void			*base;	/* vmalloc() */
		__u16			*rgb16;
		struct lookup_rgb24	*rgb24;
		struct lookup_dehuff	*dehuff;
	} table;
};
#define LUT_NULL	0
#define LUT_RGB555	1
#define LUT_RGB565	2
#define LUT_RGB24	3
#define LUT_DEHUFF4	4
#define LUT_DEHUFF5	5
#define LUT_DEHUFF6	6
#define LUT_DEHUFF7	7
#define LUT_STRETCH_H	8
#define LUT_STRETCH_V	9

struct translation
{
	int		type;
	int		width;
	int		height;
	__u8		*in;
	int		in_stride;
	__u8		*out;
	int		out_stride;
	int		output_size;
	int		output_is_user;
	int		ilut;
	struct lookup	lut[4];
};
#define XLAT_NULL		0
#define XLAT_YUYV_TO_UYVY	1
#define XLAT_YUYV_TO_YUV420	2
#define XLAT_YUYV_TO_GREY	4
#define XLAT_YUYV_TO_RGB555	6
#define XLAT_YUYV_TO_RGB565	7
#define XLAT_YUYV_TO_RGB24	8
#define XLAT_YUYV_TO_RGB32	9
#define XLAT_YCQ_TO_YUYV	10
#define XLAT_HUFF_TO_YUYV	11

/*  Per-open data for handling multiple opens on one device */
struct device_open
{
	int			isopen;
	int			noncapturing;
	struct videum_device	*dev;
};
#define MAX_OPENS	3

/*  Streaming data buffer  */
struct stream_buffer
{
	struct v4l2_q_node	qnode;
	struct v4l2_buffer	vidbuf;
	int			requested;
	__u8			*vaddress;  /* vmalloc() */
	struct scatter_node	*dma_list;  /* get_free_page() */
};
#define MAX_CAPTURE_BUFFERS	30
#define MAX_LOCKED_MEMORY	2000000

/*
 *	Videum device structure
 *
 *	One for each Videum in the system.
 *	This structure holds all the global information the drivers
 *	need about each board.
 */
struct videum_device
{
	struct v4l2_device	v;
	char			shortname[16];

/*	Per-bus index number for each Videum	*/
	int			index;
/*	General type of Videum. See defines below	*/
	int			type;
	int			wavi_type;
/*	Pointer to the pci_dev structure for this board	*/
	struct pci_dev		*pci;
/*	I/O Base address for non-PCI devices	*/
	unsigned int		iobase;
	unsigned int		cfg;
/*	I/O performance		*/
	int			slave_rate;
	int			dma_rate;
/*	Spinlock for this device to guarantee accesses are atomic	*/
	spinlock_t		lock;

/*	Copy of the board's configuration EEPROM	*/
	struct EEPROM		eeprom;

/*	Used for I2C routines */
	__u16			ctl;

/*	On-board SRAM memory map		*/
	int			sram_base;
	int			sram_size;
	int			sram_audio;
	int			sram_audio_size;
	int			sram_ucode;
	int			sram_ucode_size;
	int			sram_linebuf;
	int			sram_linebuf_size;
	int			sram_video;
	int			sram_video_size;

/*
 *	V I D E O   C A P T U R E
 */
	int			is_registered;
	int			open_count;
	struct device_open	open_data[MAX_OPENS];
	int			capturing_opens;

/*	Video Interrupts	*/
	int			irq;
	int			ints_enabled;
	struct tq_struct	tqnode_dpc;/* for BH */
	struct timer_list	tlnode;/* for polling interrupts */
	struct wait_queue	*new_video_frame;
	void			(*video_handler)(void *);

/*	Video decoder stuff		*/
	struct video_decoder	videc;

	struct video_source	source[VSOURCE_COUNT];

/*	WAVI video sampler microcode	*/
	int			uc_loaded;
	struct ucode_parms	uc_parm[2];
	__u8			uc_buffer[2][UC_SIZE];
	int			uc_length[2];

/*	Client capture format and capture modes	*/
	struct v4l2_format	clientfmt;
	struct v4l2_captureparm	capture;
	int			input;

/*	Hardware capture format	(see ucode_parms for width & height) */
	int			capture_compress;
	int			capture_vdf;
	int			capture_bpp;
	int			capture_size;
	int			capture_min_size;
	int			capture_wcase_size;
	int			capture_low_water;
	int			capture_high_water;
	int			capture_marker_spacing;

/*	Capture buffer  */
	__u8			*capture_buffer;/* vmalloc() */
	int			capture_buffer_size;
	struct scatter_node	*capture_dma_list;/* get_free_page() */

/*	Capture state	*/
	int			ready_to_capture;
	int			sampler_enabled;
	int			capture_completed;
	unsigned long		time_acquired;/* millisecond time stamp */
	int			streaming;
	struct stream_buffer	stream_buf[MAX_CAPTURE_BUFFERS];
	int			stream_buffers_requested;
	int			stream_buffers_mapped;
	int			stream_contig_map;
	struct v4l2_queue	stream_q_capture;
	struct v4l2_queue	stream_q_done;
	struct timeval		stream_begin;
	unsigned long		stream_last_frame;
	__u8			*stream_capture_buffer;

/*	DMA	*/
	int			dma_mode;
	int			dma_state;

/*	Image format conversions	*/
	struct translation	translation1;
	struct translation	translation2;
	__u8			*xlat_temp;/* vmalloc() */

/*	Performance statistics	*/
	struct v4l2_performance perf;

/*
 *	A U D I O   M I X E R
 */
	int			mixer_registered;
	int			mixer_unit;
	int			mix_devmask; /* channels with level controls */
	int			mix_stereodevs; /* ch's w/ stereo level ctrl */
	int			mix_recmask; /* record inputs */
	int			mix_source;
	int			mix_reclev;
	int			mix_imix;
	int			mix_cd;
	int			mix_pcm;
	int			mix_volume;
	int			mix_modcount;

/*
 *	A U D I O   R E C O R D / P L A Y B A C K
 */
	int			dsp_registered;
	int			dsp_unit;
	int			a_open;
	int			a_open_mode;
	int			a_format;  /* always 16-bit signed  */
	int			a_stereo;  /* 0 = mono;  1 = stereo */
	int			a_speed;   /* sample rate in Hz     */
	int			a_xtal;
	int			a_freq;
	int			a_last_audptr;

	__u8			*a_record_buffer;
	__u8			*a_record_read_ptr;
	__u8			*a_record_xfer_ptr;
	int			a_record_data_avail;
	int			a_record_audptr;
	int			a_record_sram_data;
	int			a_do_record_xfers;
	struct wait_queue	*a_record_wait;

	__u8			*a_play_buffer;
	__u8			*a_play_write_ptr;
	__u8			*a_play_xfer_ptr;
	int			a_play_space_avail;
	int			a_play_audptr;
	int			a_play_sram_avail;
	int			a_do_play_xfers;
	struct wait_queue	*a_play_wait;

	int			a_dma_enabled;
	int			a_fragment;
	int			a_total_bytes;     /* total played/recorded */
	int			a_total_fragments; /* total played/recorded */

	struct tq_struct	a_tqnode_dpc;/* for BH */
	struct timer_list	a_tlnode;/* for polling */

/*	Remember state of some write-only registers	*/
	__u16			wavi_ATTN;
	__u16			wavi_GAIN;
	__u16			wavi_MIX1;
	__u16			wavi_MIX2;
	__u16			wavi_IMIBIT;  /* 0 or WAVI_IMIBIT */
};
/*	Values for type field	*/
#define VIDEUM_TYPE_BOGUS	0
#define VIDEUM_TYPE_PCI		1
#define VIDEUM_TYPE_ISA		2
#define VIDEUM_TYPE_PNPISA	3
#define WINNOV_WAVI_93		93
#define WINNOV_WAVI_95		95
#define WINNOV_WAVI_97		97
/*	Values for videc.type, the video decoder type	*/
#define VIDEC_BOGUS		0
#define VIDEC_7110		1
#define VIDEC_7111A		2
#define VIDEC_VPX		3
/*	Values for the microcode loaded status	*/
#define UC_UNDEF		(-1)
#define UC_NORMAL		0
#define UC_WORSTCASE		1
/*	DMA modes	*/
#define DMAMODE_SLAVE		0/* not using DMA */
#define DMAMODE_SIMPLE		1/* DMA to capture_buffer */
#define DMAMODE_FRAMEBUFFER	2/* DMA to external frame buffer */
#define DMA_STATE_IDLE		0
#define DMA_STATE_ARMING	1
#define DMA_STATE_SUSPENDED	2
#define DMA_STATE_RUNNING	3
/*	Extreme video dimensions	*/
#define MIN_WIDTH		32
#define MIN_HEIGHT		24
#define MAX_WIDTH		640
#define MAX_HEIGHT		288
#define MAX_FRAME_AGE		200 /* ms */
#define LOW_WATER		60 /* (%) Switch to higher quality */
#define HIGH_WATER		90 /* (%) Switch to lower quality */

/*	Macros for PCI Videum i/o port addressing	*/
#define PCIBA0(dev)		(dev->pci->base_address[0]&~1)
#define PCIBA1(dev)		(dev->pci->base_address[1]&~1)
#define PCIBA2(dev)		(dev->pci->base_address[2]&~1)
#define PCIBA3(dev)		(dev->pci->base_address[3]&~1)
#define PCIBA4(dev)		(dev->pci->base_address[4]&~1)
#define WDATA(dev)		(PCIBA2(dev)+0)
#define LBA(dev)		(PCIBA2(dev)+4)
#define JTAG(dev)		(PCIBA0(dev)+0)
#define DMAC(dev)		(PCIBA3(dev)+2)
#define BMCMD0(dev)		(PCIBA4(dev)+0)
#define MRDMODE(dev)		(PCIBA4(dev)+1)
#define BMSTATUS0(dev)		(PCIBA4(dev)+2)
#define BMPRDT0(dev)		(PCIBA4(dev)+4)
#define BMCMD1(dev)		(PCIBA4(dev)+8)
#define BMSTATUS1(dev)		(PCIBA4(dev)+10)
#define BMPRDT1(dev)		(PCIBA4(dev)+12)
#define BMCMD_RW_CNRL		(1<<3)
#define BMCMD_START_STOP	(1<<0)
#define BMSTATUS_DMA_INT	(1<<2)
#define BMSTATUS_ERROR		(1<<1)
#define BMSTATUS_ACTIVE		(1<<0)
#define DMAC_HIRQREQn		(1<<4)
#define DMAC_SWAP		(1<<3)
#define DMAC_RSTMDMA1		(1<<2)
#define DMAC_HDMAEN1		(1<<1)
#define DMAC_HIRQEN		(1<<0)


#ifndef PKMOD
#define PKMOD "wnv: "
#endif
#if 1
#define debug_msg(fmt,arg...) printk(KERN_DEBUG PKMOD fmt,##arg)
#else
#define debug_msg(fmt,arg...)
#endif
#if 1
#define err_msg(fmt,arg...) printk(KERN_ERR PKMOD fmt,##arg)
#else
#define err_msg(fmt,arg...)
#endif
#if 1
#define info_msg(fmt,arg...) printk(KERN_INFO PKMOD fmt,##arg)
#else
#define info_msg(fmt,arg...)
#endif


/*
 *	Stuff exported from wnv.c
 *
 */
/*	Global device structure  */
extern struct videum_device videum[WNV_NBOARDS];

/*	WAVI chip and on-board memory access	*/
extern __u16 wavi_readreg(struct videum_device *dev,
			  int reg);
extern __u16 wavi_writereg(struct videum_device *dev,
			   int reg, __u16 value);
extern void wavi_block_read(struct videum_device *dev,
			    int sram_address, __u8 *buffer, int length);
extern void wavi_block_write(struct videum_device *dev,
			     int sram_address,  __u8 *buffer, int length);
extern void wavi_benchmark(struct videum_device *dev);

/*	Busmaster functions  */
extern int wnv_bm_cleanup(struct videum_device *dev);
extern int wnv_bm_resync(struct videum_device *dev);
extern int wnv_bm_disarm(struct videum_device *dev);
extern int wnv_bm_arm(struct videum_device *dev,
		      struct scatter_node *list);
extern int wnv_bm_is_done(struct videum_device *dev);
extern int wnv_bm_build_scatter_list(struct videum_device *dev,
				     unsigned char *buffer,
				     struct scatter_node **plist);

/*	I2C Bus functions  */
extern void i2c_get_ready(struct videum_device *dev);
extern int wnv_i2c_read_reg_byte(struct videum_device *dev, 
				 int addr, int reg);
extern int wnv_i2c_write_reg_byte(struct videum_device *dev, 
				  int addr, int reg, int ndata);
extern int wnv_i2c_read_reg_word(struct videum_device *dev, 
				 int addr, int reg);
extern int wnv_i2c_write_reg_word(struct videum_device *dev, 
				  int addr, int reg, int ndata);

/*	Interrupt functions  */
extern int wnv_register_video_interrupt(struct videum_device *dev,
					void (*handler)(void *));
extern void wnv_unregister_video_interrupt(struct videum_device *dev);
extern void wnv_interrupt_enable(struct videum_device *dev);
extern void wnv_interrupt_disable(struct videum_device *dev);
